#!/usr/bin/perl

# Perl script to add help text to Matlab file from a corresponding LaTeX
# reference file

$file = shift(@ARGV);
open(MATIN, $file) || do {
    die "Cannot open input Matlab file, $file. Stopped";
};
$file = shift(@ARGV);
open(TEXIN, $file) || do {
    die "Cannot open input LaTeX file, $file. Stopped";
};
$file = shift(@ARGV);
$file = '>'.$file;
open(MATOUT, $file) || do {
    die "Cannot open output Matlab file, $file. Stopped";
};

@subsections=('Purpose', 'Description', 'See Also');
$see_also = 'See Also'; # Sub-section treated differently for benefit of helpwin
# LaTeX command for reference sub-section
$refsub = '\\\\refsub';
# Purpose sub-section becomes H1 help line
$purpose = 'Purpose';
$copyright_text = "\n%\tCopyright (c) Ian T Nabney (1996-2001)";


# Output file format: % <tab> 68 characters
$text_format = "%\t^" . '<' x 68;
$cont_text =  $text_format . "~~";
$paragraph = "%\n";
$form = <<"End of format definition";
format MATOUT = 
$text_format~~
\$rest
.
End of format definition

eval $form;

$fun_name_found = 0;
$script = 0;
# Read first lines from raw Matlab input
#$/ = "%";
#$* = 1;
for (;;) {
    $inline = <MATIN>;
    if ($fun_name_found == 0) {
	# Ignore blank lines
	if (/\s+/) {
	    ;
	}
	elsif ($inline !~ /.*function.*/) {
	    # This is a script
	    $script = 1;
	    last;
	}
        # Extract the function name and capitalise. Function may not
        # return a value.
        elsif ($inline =~ /[\s]*function(.+=[\s]*|[\s]*)([\w]+)[\s]*\(.*/o) {
            ($fun_name = $2) =~ tr/a-z/A-Z/;
            $fun_name_found = 1;
        };
    };

# Write line to output Matlab file
    if ($fun_name_found) {
        print MATOUT $inline;
    }
# If line does not contain a ), then carry on
    last if ($inline =~ /\)/);
};

# If function name not found, then something must have gone wrong
if ($fun_name_found == 0 && $script == 0) {
    die "No function name in Matlab file. Stopped";
};

# Set paragraph mode on input
$/ = '';

# Set output format
$~ = 'MATOUT';
# Don't want page breaks in output, so set current page length very long
$= = 500;

$print = 0;
$ref_fun_name_found = 0;
while (<TEXIN>) {
# Find function name from \refsec argument
    if ($ref_fun_name_found == 0) {
	if (/$refsec\{([\w]+)\}/) {
	    # Extract the function name and capitalize
            ($ref_fun_name = $1) =~ tr/a-z/A-Z/;
            $ref_fun_name_found = 1;
	    # Check that function name matches that in function file
	    if (!$script && $fun_name ne $ref_fun_name) {
		die "Function name in reference does not match that in code. Stopped";
	    }
	    $fun_name = $ref_fun_name;
	}
    }

    if (/$refsub\{([\w\s]+)\}/) {
	$rest = substr($_, length($refsub)+1);
	$subsec = $1;
#	chop $subsec; 
# Is this sub-section to be printed?
	$print_subsec = 0;
	foreach $sub (@subsections) {
	    if ($sub eq $subsec) {
# Now we know that this sub-section should be printed
		$print_subsec = 1;
		$print = 1;

		$rest = substr($rest, 1+length($subsec));
		# Remove any bits of LaTeX commands
		if ($sub eq $purpose) {
		    # This will give the H1 line.  Remove the Purpose heading
		    $rest =~ s/$purpose//;
	    	    # Assume that text is on one line
	    	    if (length($fun_name) > 6) {
			$rest = "%$fun_name " . $rest;
		    }
		    else {
			$rest = "%$fun_name\t" . $rest;
		    }
		    chop $rest;
		    print MATOUT $rest, $paragraph;
		}
		elsif ($subsec eq $see_also) {
		    printf MATOUT "%%\tSee also\n";
		    &write_output;
		}
		else {
		    # Print header
		    printf MATOUT "%%\t%s\n", $subsec;
		    # Write normal formatted output
		    &write_output;
		}
	    }
	}
	if ($print_subsec <= 0.5) {
# This is not a subsection to print, so turn off paragraph printing
	    $print = 0;
	}	    
    }

# This is a paragraph without a heading
    else {
	if ($print > 0.5) {
	    # Copy $_ into $rest
	    $rest = substr($_, 0);
#	    &strip_latex;
	    # Write normal formatted output
	    &write_output;
	}
    }
}

# The last of the header is the copyright notice.
# David Evans contributed largely to MDN software
if (substr($fun_name, 0, 3) eq 'MDN') {
    $copyright_text = $copyright_text . "\n%\tDavid J Evans (1998)";
}
$copyright_text = $copyright_text . "\n\n";
print MATOUT $copyright_text;
		
# Now write out the rest of the raw Matlab file
if ($script) {
    print MATOUT $inline;
}

while (<MATIN>) {
    print MATOUT $_;
}

sub strip_latex {
#    print $rest;
    # Special treatment for matlab macro
    $rest =~ s/\\matlab[\\]*/MATLAB/g;
    # Convert text in \mat to upper case: e forces evaluation of expression
    $rest =~ s/\\mat\{([^\}]+)\}/uc($1)/eg;
    # Remove any bits of LaTeX commands
    # This line strips out \begin{command}
    $rest =~ s/\\begin\{[\w]+\}//g;
    # This line strips out \end{command}
    $rest =~ s/\\end\{[\w]+\}//g;
    # This line strips out \command{ text } and leaves text
    $rest =~ s/\\[\w]+\{([^\}]+)\}/$1/g;
    # This line strips out any remaining \
    $rest =~ s/\\//g;
}

sub verblines
{
    my ($value) = @_;
    my (@outlines);
    @lines = split(/\n/, $value);
    foreach $line (@lines) {
	if (length($line) > 0) {
	    if ($subsec eq $see_also) {
		$line =~ tr/a-z/A-Z/;
	    }
	    $line = "%\t" . $line . "\n";
	}
	push @outlines, $line;
    }
    print MATOUT @outlines;
}

sub write_output
{
    ($temp, @segs) = split(/\\begin\{verbatim\}/, $rest);
    $rest = $temp;
    &strip_latex;
    write(MATOUT);
    foreach $seg (@segs) {
	($verb, $rest) = split(/\\end\{verbatim\}/, $seg);
	verblines($verb);
	&strip_latex;
	write(MATOUT);
    }
    print MATOUT $paragraph;
}
