
function textdisplay(varargin)

% TEXTDISPLAY(VARARGIN) to dispay text in a plot.
%	text: is the input text
%	pos: is the input text position
%
%	Copyright (c) Davide D'Alimonte (2007)
%

% Check that the number of input parameters is correct
if nargin == 0
   disp('TEXTDISPLAY ERROR: input parameters are missing in the textdisplay call.')
   return
elseif nargin/2 ~= floor(nargin/2)
   disp('TEXTDISPLAY ERROR: the number of input parameters is wrong in textdisplay.')
   return
end

% Check that the first input parameter is a string
if ~isstr(varargin{1})
   disp('TEXTDISPLAY ERROR: the first input parameter is not a string.')
   return
end

ax = gca;
v = axis;
par.pos = [(v(2)-v(1))/15, v(4)-(v(4)-v(3))/15];

% Default text setting
for i = 1:2:nargin
   par.(varargin{i}) = varargin{i+1};
   if i == nargin - 2
      break
   end
end

% Write a dummy annotation to get the text position
hold on;
hTxt = text(par.pos(1), par.pos(2), par.text);
set(hTxt, 'VerticalAlignment', 'top');

% Apply a patch over the text extension
ex = get(hTxt, 'ex');
delete(hTxt);
x = [.9*ex(1) .9*ex(1) ex(1)+1.1*ex(3) ex(1)+1.1*ex(3)];
y = [.95*ex(2) ex(2)+1.05*ex(4) ex(2)+1.05*ex(4) .95*ex(2)];
hPatch = patch(x, y, get(gca, 'color'));

% Write the text over the patch
hTxt = text(par.pos(1), par.pos(2), par.text);
set(hTxt, 'VerticalAlignment', 'top');
axis(v);
